@@ -7,13 +7,6 @@ import (
 	"golang.org/x/sys/unix"
 )
 
-const (
-	// For incoming connections.
-	TCP_FASTOPEN = 23
-	// For out-going connections.
-	TCP_FASTOPEN_CONNECT = 30
-)
-
 func bindAddr(fd uintptr, ip []byte, port uint32) error {
 	setReuseAddr(fd)
 	setReusePort(fd)
@@ -59,8 +52,8 @@ func applyOutboundSocketOptions(network string, address string, fd uintptr, conf
 			tfo = 1
 		}
 		if tfo >= 0 {
-			if err := syscall.SetsockoptInt(int(fd), syscall.SOL_TCP, TCP_FASTOPEN_CONNECT, tfo); err != nil {
-				return newError("failed to set TCP_FASTOPEN_CONNECT=", tfo).Base(err)
+			if err := syscall.SetsockoptInt(int(fd), syscall.SOL_TCP, unix.TCP_FASTOPEN_CONNECT, tfo); err != nil {
+				return newError("failed to set TCP_FASTOPEN_CONNECT", tfo).Base(err)
 			}
 		}
 
@@ -95,6 +88,25 @@ func applyOutboundSocketOptions(network string, address string, fd uintptr, conf
 				return newError("failed to set TCP_WINDOW_CLAMP", err)
 			}
 		}
+
+		if config.TcpUserTimeout > 0 {
+			if err := syscall.SetsockoptInt(int(fd), syscall.IPPROTO_TCP, unix.TCP_USER_TIMEOUT, int(config.TcpUserTimeout)); err != nil {
+				return newError("failed to set TCP_USER_TIMEOUT", err)
+			}
+		}
+
+		if config.TcpMaxSeg > 0 {
+			if err := syscall.SetsockoptInt(int(fd), syscall.IPPROTO_TCP, unix.TCP_MAXSEG, int(config.TcpMaxSeg)); err != nil {
+				return newError("failed to set TCP_MAXSEG", err)
+			}
+		}
+
+		if config.TcpNoDelay {
+			if err := syscall.SetsockoptInt(int(fd), syscall.IPPROTO_TCP, unix.TCP_NODELAY, 1); err != nil {
+				return newError("failed to set TCP_NODELAY", err)
+			}
+		}
+
 	}
 
 	if config.Tproxy.IsEnabled() {
@@ -115,8 +127,8 @@ func applyInboundSocketOptions(network string, fd uintptr, config *SocketConfig)
 	if isTCPSocket(network) {
 		tfo := config.ParseTFOValue()
 		if tfo >= 0 {
-			if err := syscall.SetsockoptInt(int(fd), syscall.SOL_TCP, TCP_FASTOPEN, tfo); err != nil {
-				return newError("failed to set TCP_FASTOPEN=", tfo).Base(err)
+			if err := syscall.SetsockoptInt(int(fd), syscall.SOL_TCP, unix.TCP_FASTOPEN, tfo); err != nil {
+				return newError("failed to set TCP_FASTOPEN", tfo).Base(err)
 			}
 		}
 
@@ -151,6 +163,18 @@ func applyInboundSocketOptions(network string, fd uintptr, config *SocketConfig)
 				return newError("failed to set TCP_WINDOW_CLAMP", err)
 			}
 		}
+
+		if config.TcpUserTimeout > 0 {
+			if err := syscall.SetsockoptInt(int(fd), syscall.IPPROTO_TCP, unix.TCP_USER_TIMEOUT, int(config.TcpUserTimeout)); err != nil {
+				return newError("failed to set TCP_USER_TIMEOUT", err)
+			}
+		}
+
+		if config.TcpMaxSeg > 0 {
+			if err := syscall.SetsockoptInt(int(fd), syscall.IPPROTO_TCP, unix.TCP_MAXSEG, int(config.TcpMaxSeg)); err != nil {
+				return newError("failed to set TCP_MAXSEG", err)
+			}
+		}
 	}
 
 	if config.Tproxy.IsEnabled() {
